BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ACCESS_REVIEW_DETAILS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ACCESS_REVIEW_DETAILS_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ACCESS_REVIEW_DETAILS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ACCESS_REVIEW_DETAILS_V';
  END LOOP;
END;
/

CREATE VIEW access_review_details_v AS
   SELECT DISTINCT
    item.id review_item_id,
    item.last_override_action_id last_override_action,
    instance.id review_instance_id,
    instance.status review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    action.last_update,
    item.approved_time,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids reviewer_id,
      CASE
        WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN TO_CHAR(final_reviewer.display_name)
        ELSE TO_CHAR(multi_reviewer.reviewers)
      END reviewer_display_name,
      CASE
        WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN TO_CHAR(substr(final_reviewer.dn, 4, instr(final_reviewer.dn, ',', 1, 1)-4))
        ELSE TO_CHAR(multi_reviewer.user_cn)
      END reviewer_cn,
    reassigned_by_history.reviewers reassigned_by_user_disp_name,
    reassigned_from_history.reviewers reassigned_from_user_disp_name,
    reassigned_by_history.reassigned_date,
    reassigned_by_history.reasons reassigned_reasons,
    reassigned_by_history.rev_comments reassigned_comments,
    defn.name review_name,
    TO_CHAR(substr(defn.description,0,4000)) description,
    defn.review_type,
    defn.next_schedule_date,
      CASE
        WHEN item.item_type = 'ACCOUNT' AND usr.id IS NULL THEN shacc.suser
        ELSE usr.id
      END user_id,
      CASE
        WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN CAST(shacc.display_name as nvarchar2(2000))
        ELSE CAST(usr.display_name as nvarchar2(2000))
      END user_display_name,
      CASE
        WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN TO_CHAR(shacc.user_cn)
        ELSE TO_CHAR(substr(usr.dn, 4, instr(usr.dn, ',', 1, 1)-4))
      END user_cn,
    action.action reviewer_action,
    reason.name modify_reason_name,
    reason.id modify_reason_id,
      CASE
        WHEN fulfillment.review_item_id IS NOT NULL THEN 1
        ELSE 0
      END remove_completed,
    usr.unique_user_id,
    action.review_order,
    action.reviewer_comment,
    perm.id permission_id,
    perm.name permission_name,
    acct.id account_id,
    acct.account_id account_native_id,
    acct.name account_name,
    acct.relation_to_user_type account_relation,
    TO_CHAR(app.id) application_id,
    TO_CHAR(app_ds.name) application_name,
    item.final_decision,
    item.final_decision_time,
    role.id role_id,
    role.name role_name,
    item.item_type,
    brole.name brole_name,
    brole.id business_role_id,
    action.fulfillment_instructions,
    esc_reviewer.display_name esc_reviewer,
    sup.unique_user_id supervisor_id,
    sup.display_name supervisor_display_name
  FROM review_item item
    JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
    JOIN (SELECT
      review_instance.id inst_id,
      defn.id defn_id,
      CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
      CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END AS description,
      CASE WHEN staged_defn.next_schedule_date IS NULL THEN defn.next_schedule_date ELSE staged_defn.next_schedule_date END AS next_schedule_date,
      defn.review_type,
      defn.deleted
      FROM review_definition defn
      JOIN review_instance ON defn.id = review_instance.review_definition
      LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
    LEFT JOIN suser usr ON usr.unique_user_id = item.unique_user_id AND usr.effective = 1
    LEFT JOIN suser sup ON sup.unique_user_id = item.unique_supervisor_id AND sup.effective = 1
    LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN application app ON app.id =
      CASE
        WHEN perm.id IS NULL THEN acct.application
        ELSE perm.application
      END
    LEFT JOIN business_role brole ON brole.id = item.business_role_id
    LEFT JOIN role_policy role ON role.id = item.role_id
    LEFT JOIN data_source app_ds ON app_ds.id = app.id
    LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action <> 'REASSIGNED'
    LEFT JOIN review_item_action_reason reason ON reason.id = action.reason_id
    LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type = 'RO'
    LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
    LEFT JOIN ( SELECT DISTINCT
      item.id item_id,
      LISTAGG((TO_CHAR(multi_reviewers.id)), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY multi_reviewers.display_name) reviewer_ids,
      LISTAGG((TO_CHAR(multi_reviewers.unique_user_id)), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY multi_reviewers.display_name) reviewer_unique_ids,
      LISTAGG((TO_CHAR(multi_reviewers.display_name)), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY multi_reviewers.display_name) reviewers,
      LISTAGG((TO_CHAR(substr(multi_reviewers.dn, 4, instr(multi_reviewers.dn, ',', 1, 1)-4))), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY multi_reviewers.display_name) user_cn
      FROM review_item item
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      GROUP BY item.id) multi_reviewer ON multi_reviewer.item_id = item.id
    LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id = final_reviewer.unique_user_id AND final_reviewer.effective = 1
    LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id = acting_reviewer.unique_user_id AND acting_reviewer.effective = 1
    LEFT JOIN (SELECT
      ria.review_item_id,
      LISTAGG(TO_CHAR(usr.display_name), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY ria.assigned) AS reviewers
      FROM  review_item_action ria
      LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
      LEFT JOIN suser usr ON ri.single_reviewer_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
    LEFT JOIN (SELECT
      ria.review_item_id,
      LISTAGG(TO_CHAR(usr.display_name), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY ria.assigned) AS reviewers,
      LISTAGG(COALESCE(TO_CHAR(reason.name), 'NULL_REASON'), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY ria.assigned) AS reasons,
      LISTAGG(TO_CHAR(CAST(ria.assigned AS DATE), 'YYYY-MM-DD HH24:MI:SS'), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY ria.assigned) AS reassigned_date,
      LISTAGG(TO_CHAR(COALESCE (NULLIF(TO_CHAR(ria.reviewer_comment), ''), 'NULL_COMMENT')), ' | ' on overflow truncate) WITHIN GROUP (ORDER BY ria.assigned) AS rev_comments
      FROM  review_item_action ria
      LEFT JOIN review_item_action_reason reason ON ria.reason_id = reason.id
      LEFT JOIN suser usr ON ria.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
    LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id = acct.unique_account_id
    LEFT JOIN ( SELECT DISTINCT
      defn_1.name review_name,
      ri.id rid,
      LISTAGG((TO_CHAR(s.display_name)), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY s.display_name) review_owners
      FROM review_instance ri
      JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
      LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
      LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
      LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id
    LEFT JOIN (SELECT
      esc_reviewers.review_instance_id,
      LISTAGG(TO_CHAR(esc_reviewers.display_name), ' | ' on overflow truncate) WITHIN GROUP (ORDER BY esc_reviewers.display_name) display_name
      FROM
        (SELECT DISTINCT
        TO_CHAR(u.display_name) display_name,
        rri.review_instance_id
        FROM review_item ri
        JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO'
        JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
        JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = 1) esc_reviewers
      GROUP BY esc_reviewers.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
    LEFT JOIN ( SELECT
      review_item_id,
      action,
      change_item_status
      FROM change_request_item cri
      LEFT JOIN change_item_action cia ON cri.latest_action = cia.id
      WHERE cri.change_request_type LIKE 'REMOVE' || '%' AND action = 'FULFILLED' AND (change_item_status = 'VERIFIED' OR change_item_status = 'COMPLETED')) fulfillment ON item.id = fulfillment.review_item_id

  WHERE
    defn.deleted = 0
    AND (acting_reviewer.unique_user_id LIKE '%' || multi_reviewer.reviewer_unique_ids || '%' OR acting_reviewer.unique_user_id IS NULL
      AND (action.review_order =
      ( SELECT DISTINCT min(act.review_order) min
        FROM review_item_action act
        WHERE (act.action = 'PENDING' OR act.action = 'ESCALATE') AND act.review_item_id = item.id)
      OR action.acted_by_policy = 1)
      OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = 1)
    AND (action.action <> 'PENDING' OR item.final_decision IS NULL);

GRANT SELECT ON igops.access_review_details_v TO ig_report_role;
CREATE PUBLIC SYNONYM access_review_details_v FOR igops.access_review_details_v;